"""
セレクタ機能の初期化とパッチの適用
"""

def apply_selector_patch():
    """
    セレクタ機能のパッチを適用
    server.pyでインポートして実行する
    """
    # server_patch モジュールをインポート
    try:
        # server.pyの関数をserver_patch内の関数で置き換え
        import burnworks_mcp_server_fetch.server as server
        from burnworks_mcp_server_fetch.server_patch import (
            extract_content_from_html,
            fetch_url,
            validate_fetch_args,
            get_selector_info,
            SELECTOR_SUPPORT
        )
        
        # 元の関数を拡張版で置き換える
        server.extract_content_from_html = extract_content_from_html
        server.fetch_url = fetch_url
        
        # パッチされたことを示す属性を追加
        server.SELECTOR_SUPPORT = SELECTOR_SUPPORT
        server.validate_fetch_args = validate_fetch_args
        server.get_selector_info = get_selector_info
        
        return True
    except ImportError:
        return False



"""
HTML要素セレクタを使用してコンテンツを抽出するための拡張モジュール
"""
from typing import Optional, Tuple
import logging

from bs4 import BeautifulSoup
import markdownify

logger = logging.getLogger(__name__)

def extract_with_selector(
    html: str, selector: str, selector_type: str
) -> Tuple[bool, str]:
    """
    セレクタを使用して特定のHTML要素を抽出し、マークダウンに変換する

    Args:
        html: 元のHTML文字列
        selector: 抽出対象の要素を指定するセレクタ
        selector_type: セレクタの種類 ('css', 'id', 'element')

    Returns:
        Tuple[bool, str]: (抽出成功したか, 抽出したコンテンツのマークダウン)
    """
    if not selector or not selector_type:
        return False, ""

    try:
        soup = BeautifulSoup(html, 'html.parser')
        
        if selector_type == 'id':
            # ID指定の場合
            selected_content = soup.find(id=selector)
        elif selector_type == 'element':
            # 要素名指定の場合
            selected_content = soup.find(selector)
        elif selector_type == 'css':
            # CSSセレクタ指定の場合
            selected_content = soup.select_one(selector)
        else:
            # 不明なセレクタタイプの場合
            return False, f"<error>Unknown selector type: {selector_type}. Use 'id', 'element', or 'css'</error>"
        
        if selected_content:
            content = str(selected_content)
            markdown = markdownify.markdownify(
                content,
                heading_style=markdownify.ATX,
            )
            return True, markdown
        else:
            return False, f"<error>No content found matching the {selector_type} selector: {selector}</error>"
    
    except Exception as e:
        logger.exception(f"Error extracting content with selector: {str(e)}")
        return False, f"<error>Error extracting content with selector: {str(e)}</error>"


def validate_selector_params(selector: Optional[str], selector_type: Optional[str]) -> Optional[str]:
    """
    セレクタパラメータのバリデーション

    Args:
        selector: セレクタ文字列
        selector_type: セレクタタイプ

    Returns:
        Optional[str]: エラーメッセージ。問題がなければNone
    """
    if selector and not selector_type:
        return "If 'selector' is provided, 'selector_type' must also be provided (valid types: 'css', 'id', 'element')"
            
    if selector_type and selector_type not in ['css', 'id', 'element']:
        return "'selector_type' must be one of: 'css', 'id', 'element'"
    
    return None



"""
元のMCPサーバーを拡張して、セレクタ機能を追加するためのパッチモジュール
"""
import logging
from typing import Optional, Tuple, Any

from pydantic import Field
from mcp.types import ErrorData, INVALID_PARAMS
from mcp.shared.exceptions import McpError

from burnworks_mcp_server_fetch.server import (
    extract_content_from_html as original_extract_content_from_html,
    fetch_url as original_fetch_url,
    Fetch,
)

# セレクタ機能のインポート
try:
    from burnworks_mcp_server_fetch.selector_extractor import extract_with_selector, validate_selector_params
    SELECTOR_SUPPORT = True
    logging.info("Selector support is enabled")
except ImportError:
    SELECTOR_SUPPORT = False
    logging.warning("Selector support is disabled. Install beautifulsoup4 to enable it.")

# Fetchモデルの拡張
if SELECTOR_SUPPORT:
    # selectorとselector_typeフィールドを追加
    Fetch.model_fields["selector"] = Field(
        default=None,
        description="CSS selector, ID, or element name to extract specific content",
    )
    Fetch.model_fields["selector_type"] = Field(
        default=None,
        description="Type of selector: 'css', 'id', or 'element'",
    )


# 拡張版のextract_content_from_html関数
def enhanced_extract_content_from_html(html: str, selector: Optional[str] = None, selector_type: Optional[str] = None) -> str:
    """拡張版: HTML内容を抽出してマークダウン形式に変換"""
    # セレクタ機能が利用可能でセレクタが指定されている場合
    if SELECTOR_SUPPORT and selector and selector_type:
        success, content = extract_with_selector(html, selector, selector_type)
        if success:
            return content
        # 失敗した場合は元の方法にフォールバック
    
    # 元の関数の処理を利用
    return original_extract_content_from_html(html)


# 拡張版のfetch_url関数
async def enhanced_fetch_url(
    url: str, user_agent: str, force_raw: bool = False, proxy_url: Optional[str] = None,
    selector: Optional[str] = None, selector_type: Optional[str] = None
) -> Tuple[str, str]:
    """拡張版: URLを取得して内容を返す"""
    # 元の関数を呼び出して基本的な内容を取得
    content, prefix = await original_fetch_url(url, user_agent, force_raw, proxy_url)
    
    # HTMLコンテンツでraw出力が要求されていない場合のみセレクタ処理を適用
    is_page_html = not prefix  # prefixが空なら、HTMLとして処理された
    
    if SELECTOR_SUPPORT and is_page_html and not force_raw and selector and selector_type:
        # セレクタを使用して内容を抽出
        success, selected_content = extract_with_selector(content, selector, selector_type)
        if success:
            return selected_content, prefix
    
    return content, prefix


# 元の関数をパッチ関数に置き換える
extract_content_from_html = enhanced_extract_content_from_html
fetch_url = enhanced_fetch_url


# call_tool関数でセレクタパラメータの検証を行うヘルパー関数
def validate_fetch_args(args: Any) -> None:
    """Fetch引数のバリデーション"""
    if SELECTOR_SUPPORT:
        selector = getattr(args, "selector", None)
        selector_type = getattr(args, "selector_type", None)
        
        error_msg = validate_selector_params(selector, selector_type)
        if error_msg:
            raise McpError(ErrorData(code=INVALID_PARAMS, message=error_msg))


# セレクタ情報を取得するヘルパー関数
def get_selector_info(args: Any) -> str:
    """セレクタ情報の文字列を作成"""
    if not SELECTOR_SUPPORT:
        return ""
    
    selector = getattr(args, "selector", None)
    selector_type = getattr(args, "selector_type", None)
    
    if selector and selector_type:
        return f" (extracted using {selector_type} selector: {selector})"
    return ""
